% [copt,xopt,Zopt,info] = min_sdppack(F0,Fi,blk,c,opt,x0,Z0)
%
% Solves the primal semidefinite program:
%      Minimize  c'*x      subject to  x1*Fi{1} + .... + xn*Fi{n} < F0  
% and its dual:
%      Maximize  -Tr[F0*Z] subject to  Z=Z' >0 
%                                      -Tr[Fi{k}*Z]=c(k), k=1,...,n
% x is the vector of decision variables and Z is the symmetric matrix
% of dual variables. The function converts from the canonical format 
% to SDPpack format and then solves the LMI using SDPpack.
%
% Input:
%  F0        m by m symmetric matrix
%  FI        n by 1 cell array of m by m symmetric matrices
%  BLK       A vector containing the dimensions of the 
%               blocks in F0 and Fi.  If F0 and Fi have no block
%               structure, blk = m.  
%  C         m by 1 cost vector 
%  OPT       optional: The key options which can be set are:
%               (1) opt.prt:  When prt=0, the trace of execution is
%                   turned off 
%               (2) opt.abstol: Desired absolute accuracy 
%               (3) opt.reltol: Desired relative accuracy
%               (4) opt.maxiter:  Maximum number of iterations 
%            Note 1: This is only a subset of the options for
%               SDPpack. The SDPpack manual lists many other
%               options which can be set ast follows:  If
%               the SDPpack manual refers to the option 'opt.foo',
%               set this option with 'opt.foo'.
%            Note 2: Advanced options take precedence. For
%               example, setting opt.prt and opt.prtlevel has the
%               same effect (see SDPpack manual).  In this case
%               the advanced option, opt.prtlevel takes precedence.
%            Note 3: This function implements one more option
%               which is not explicit in the SDPpack manual. If
%               the LMI has blocks which are sparse, set 
%               opt.spblocks=1, but if all blocks of
%               the lmi are full, set opt.spblocks = 0;
%               (default: opt.spblocks = 1)
%  X0        optional: Initial primal variable
%  Z0        optional: Initial dual variable
%
% Output:
%  COPT      2x1 vector, COPT(1) is the final primal cost, c'*x,
%            and COPT(2) is the final dual cost, -Tr[F0*Z]
%  XOPT      final value of the vector x of primal variables.
%  ZOPT      final value of dual variables
%  INFO      Additional information from SDPpack:
%               (1) info.iter: number of iterations
%               (2) info.compval: complementarity at each iteration
%               (3) info.objval: primal/dual objectives at each iteration
%               (4) info.feasval: primal/dual feasibility at each iteration
%               (5) info.termflag: termination condition. If info.iter>0,
%                   some key flags are:
%                     termflag = -2,-1 ==> some variables are blowing up
%                     termflag = 0 ==> success
%                     termflag = 6 ==> exceeded maximum # of iterations
%                     termflag = 7 ==> data failed validation test
%               (6) info.solvertime:  Not an SDPpack output. This is the
%                cputime actually spent in the SDPpack optimization.
%            See SDPpack manual for more details. 
%
% See also: 
%
% Coding:   6/30/2001   Pete Seiler

function [copt,xopt,Zopt,info] = min_sdppack(F0,Fi,blk,c,opt,x0,Z0)

% Is function called correctly?
if nargin <4 | nargout > 4
  error(['usage: [copt,xopt,Zopt,info] = ' ...
    'min_sdppack(F0,Fi,blk,c,{opt},{x0},{Z0})']);
end
if nargin < 7
  Z0=[];
  if nargin <6
    x0=[];
    if nargin <5
      opt = [];
    end
  end
end

% Is SDPpack installed and in Matlab's search path?
if exist('fsql','file')~=2
  error( sprintf(['SDPpack is not in Matlab''s search path. \n' ... 
   'To download SDPpack, go to: \n' ...
   '\t http://cs.nyu.edu/cs/faculty/overton/sdppack/sdppack.html' ]) );
end

% Store input variables
F0in = F0;   Fiin = Fi;   blkin = blk;
optin = opt; x0in = x0;   Z0in = Z0;

% Convert from canonical form to SDPpack form
ndec = size(Fiin,1); base =0;
Fifull = cell(ndec,1);
Fi.s = [];Fi.l=[];
F0.s = [];F0.l=[];
blk.s=[];blk.l=0;
for i1=1:length(blkin)  
  idx = base+[1:blkin(i1)];
  if blkin(i1)==1 % store linear constraints separately
    for i2 = 1:ndec;
      temp(i2,1) = Fiin{i2}(idx,idx);
    end
    Fi.l=sparse([Fi.l temp]);
    F0.l=[F0.l; F0in(idx,idx)];
    blk.l = blk.l+1;
  else
    for i2=1:ndec;  
      Fifull{i2} = sparse(blkdiag( Fifull{i2},Fiin{i2}(idx,idx) ));
    end
    F0.s=sparse( blkdiag(F0.s,F0in(idx,idx)) );
    blk.s = [blk.s, blkin(i1)];   
  end  
  base = base+blkin(i1);
end    
% Set spblocks=1 if blocks of lmi are sparse
spblocks = 1;
if isfield(optin,'spblocks');  spblocks = optin.spblocks;   end
Fi.s = makeA(Fifull,blk.s,spblocks);

% Set SDPpack options (setopt is an SDPpack m-file)
blk.q=[];
if ~optin.prt; warning off; end;
setopt; 
fldnames = fieldnames(optin);
for i1 =1:length(fldnames)
  eval(['opt.' fldnames{i1} '= optin.' fldnames{i1} ';']);
end
if ~isfield(optin,'prtlevel'); opt.prtlevel = optin.prt;     end
if ~isfield(optin,'maxit');    opt.maxit = optin.maxiter;    end

% Set initial conditions (init is an SDPpack m-file)
b=-c;  
init;

% Solve with SDPpack (some code stripped from sql.m, an
%   SDPpack m-file which calls fsql.m)
tau_orig = opt.tau;
scalefac_orig = scalefac;
cputime0 = cputime;
for runs = 1:3
   [xdual,xopt,slackvar,iter,compval,feasval,objval,termflag] = ...
        fsql(Fi,-c,F0,blk,X,y,Z,opt);
   if iter == 0
      if termflag == 4 | termflag == 5
         fprintf('\nsql: Initial point may be too close to boundary\n');
      elseif termflag == 6 & opt.maxit == 0
         fprintf('\nsql: Data passed validation tests\n');
      elseif termflag == 7 & opt.prtlevel == 0
         fprintf('\nsql: Data failed validation tests\n');
         fprintf('For more information, rerun with prtlevel = 1\n');
      end
      break;
   end
  if autorestart & (termflag == 4 | termflag == 5)...
     & (compval(iter+1) + feasval(iter+1,1) + feasval(iter+1,2) > 5.0e-3)
      if runs < 3
         fprintf('\nrestarting...\n');
      end
      if runs == 1
         opt.tau = min(opt.tau,.99);
      elseif runs == 2
         opt.tau = min(opt.tau,.9);
      end
      scalefac = 100*scalefac;
      init;
   else
      break
   end
end    % runs
info.solvertime = cputime-cputime0;
warning on;
info.iter = iter;
info.compval = compval;
info.feasval = feasval;
info.objval = objval;
info.termflag = termflag;


% Convert dual variables back to matrix form
base.s =0;base.l = 1;Zopt = [];
for i1=1:length(blkin)  
  if blkin(i1)==1 % recover linear constraints separately
    Zopt = blkdiag(Zopt,xdual.l(base.l));
    base.l = base.l +1;
  else
    idx = base.s+[1:blkin(i1)];
    Zopt = blkdiag(Zopt,xdual.s(idx,idx));
    base.s = base.s + blkin(i1);
  end
end
Zopt = full(Zopt);
copt = [c'*xopt -trace(F0in*Zopt)];



