% [copt,xopt,Zopt,info] = minsdp(solver,lmisys,c,opt)
%
% Solves the primal semidefinite program:
%      Minimize  c'*x      subject to  x1*F1+...+xn*Fn < F0
% and its dual:
%      Maximize  -Tr[F0*Z] subject to  Z=Z' >0 
%                                      -Tr[Fi*Z]=c(i), i=1,...,n
% x is the vector of decision variables and Z is the symmetric matrix
% of dual variables. The LMI constraints of the primal problem are 
% specified by the vector LMISYS which must be a LMILAB description 
% of an LMI system.  The function converts from LMILAB format to a 
% canonical format and then solves the LMI using software specified by 
% SOLVER.
%
% Input:
%  SOLVER    A string which specifies the solver to be used.
%               Valid choices: sdpha, sdppack, sdpt3, sedumi, or sp
%  LMISYS    LMILAB description of the system of LMI constraints
%  C         vector of the same size as  x 
%  OPT       optional: The key options which can be set are:
%               (1) opt.prt:  When prt=0, the trace of execution is
%                   turned off (default: prt=1).
%               (2) opt.abstol: Desired absolute accuracy 
%                   (default: abstol=1e-4)
%               (3) opt.reltol: Desired relative tolerance 
%                   (default: reltol=1e-4)
%               (4) opt.maxiter:  Maximum number of iterations 
%                   (default: maxiter = 100)
%               Note 1: This is only a subset of the options for each
%               solver.  For advanced options, consult solver manual
%               and type 'help min_foo' to see how to set these
%               additional options for the solver 'foo'.
%               Note 2: reltol will be ignored by sedumi and abstol
%               will be ignored by sdpha and sdpt3.  prt will be
%               ignored by sp.
%
% Output:
%  COPT      2x1 vector, COPT(1) is the final primal cost, c'*x,
%            and COPT(2) is the final dual cost, -Tr[F0*Z]
%  XOPT      final value of the vector x of primal variables.
%               Use DEC2MAT to get the corresponding matrix variable
%               values.
%  ZOPT      final value of dual variables
%  INFO      Additional solver dependent information.  This always
%            contains 'info.solvertime', which is the cputime spent
%            in the particular solver.  All time spent converting
%            from LMILab to the solver format is not counted. Type
%            'help min_foo' to see additional INFO given by the 
%            solver 'foo'.  
%
% See also: mincx, dec2mat, feasp
%
% Coding:   6/29/2001   Pete Seiler

function [copt,xopt,Zopt,info] = minsdp(solver,lmisys,c,opt)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Error Checking
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
% Is function called correctly?
if nargin <3 | nargout >4
  error(['usage: [copt,xopt,Zopt,info] = ' ...
     'minsdp(solver,lmisys,c,{opt},{x0},{Z0})']);
end
if nargin < 4
  opt = [];
end

% Set Initial Conditions to empty values: Not currently used.
x0=[];
Z0=[];


% Set default options
if ~isfield(opt,'prt');     opt.prt = 1;        end
if ~isfield(opt,'abstol');  opt.abstol = 1e-4;  end
if ~isfield(opt,'reltol');  opt.reltol = 1e-4; end
if ~isfield(opt,'maxiter'); opt.maxiter = 100;  end

% Is SOLVER a string?
if ~ischar(solver) 
  error('The input SOLVER must be a string');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Convert to Canonical form: 
%    x1*Fi{1} + ... + xn*Fi{n} < F0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[F0,Fi,blk] = lmitrans(lmisys);

% Is c a column vector of same dimension as x?
c=c(:); % Make c a column vector
if length(c)~=size(Fi,1) | min(size(c))~=1
  error(['The vector C should be of length ',num2str(size(Fi,1))]);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Use appropriate solver to solve the minimization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if strcmpi(solver,'sdpha'),
  if opt.prt 
    fprintf('\n\t --------- You are using SDPHA ---------\n\n'); 
  end
  [copt,xopt,Zopt,info] = min_sdpha(F0,Fi,blk,c,opt,x0,Z0);
    
elseif strcmpi(solver,'sdppack'),
  if opt.prt
    fprintf('\n\t --------- You are using SDPpack ---------\n\n'); 
  end
  [copt,xopt,Zopt,info] = min_sdppack(F0,Fi,blk,c,opt,x0,Z0);
  
elseif strcmpi(solver,'sdpt3'),
  if opt.prt
    fprintf('\n\t --------- You are using SDPT3 ---------\n\n'); 
  end
  [copt,xopt,Zopt,info] = min_sdpt3(F0,Fi,blk,c,opt,x0,Z0);
  
elseif strcmpi(solver,'sedumi'),
  if opt.prt 
    fprintf('\n\t --------- You are using Sedumi ---------\n\n'); 
  end
  [copt,xopt,Zopt,info] = min_sedumi(F0,Fi,blk,c,opt,x0,Z0);
  
elseif strcmpi(solver,'sp'),
  if opt.prt 
    fprintf('\n\t --------- You are using SP ---------\n\n'); 
  end
  [copt,xopt,Zopt,info] = min_sp(F0,Fi,blk,c,opt,x0,Z0);
  
else
  error( sprintf(['The input SOLVER must be one of ' ... 
	'the following strings: \n' ...
	'\t sdpha, sdppack, sdpt3, sedumi, or sp']) ); 
end


