% [copt,xopt,Zopt,info] = min_sdpt3(F0,Fi,blk,c,opt,x0,Z0)
%
% Solves the primal semidefinite program:
%      Minimize  c'*x      subject to  x1*Fi{1} + .... + xn*Fi{n} < F0  
% and its dual:
%      Maximize  -Tr[F0*Z] subject to  Z=Z' >0 
%                                      -Tr[Fi{k}*Z]=c(k), k=1,...,n
% x is the vector of decision variables and Z is the symmetric matrix
% of dual variables. The function solves the LMI using SDPT3.
%
% Input:
%  F0        m by m symmetric matrix
%  FI        n by 1 cell array of m by m symmetric matrices
%  BLK       A vector containing the dimensions of the 
%               blocks in F0 and Fi.  If F0 and Fi have no block
%               structure, blk = m.  
%  C         m by 1 cost vector 
%  OPT       optional: The key options which can be set are:
%               (1) opt.prt:  When prt=0, the trace of execution is
%                   turned off 
%               (2) opt.abstol: NOT USED
%               (3) opt.reltol: Desired relative accuracy 
%               (4) opt.maxiter:  Maximum number of iterations 
%               Note 1: This is only a subset of the options for
%               SDPT3. The SDPT3 manual lists many other
%               options which can be set as follows:  If
%               the SDPT3 manual refers to the option 'OPTIONS.foo',
%               set this option with 'opt.foo'.
%               Note 2: Advanced options take precedence. For
%               example, setting opt.prt and OPTIONS.printyes has 
%               the same effect (see SDPT3 manual).  In this case
%               the advanced option, OPTIONS.printyes takes precedence.
%               Note 3: This function has an extra option which is
%               not explicit in the SDPT3 manual.  To solve the problem
%               with the infeasible path algorithm (i.e. sdp.m), set
%               opt.alg='sdp'.  To solve the problem with the 
%               homogeneous self dual algorithm (i.e. sdphlf.m), set 
%               opt.alg='sdphlf'. (Default: opt.alg = 'sdp');
%  X0        optional: Initial primal variable
%  Z0        optional: Initial dual variable
%
% Output:
%  COPT      2x1 vector, COPT(1) is the final primal cost, c'*x,
%            and COPT(2) is the final dual cost, -Tr[F0*Z]
%  XOPT      final value of the vector x of primal variables.
%  ZOPT      final value of dual variables
%  INFO      Additional information from SDPT3:
%            (1) info.gaphist: history of primal-duality gap 
%            (2) info.infeashist: history of infeasibility measure
%            (3) info.info: 5x1 vector containing the following:
%                   [termination code, iterations, final gap, 
%                        infeasibility measure, cputime]
%                info(1)=0 for normal termination
%            (4) info.solvertime: Not an SDPT3 output.  This is the
%                cputime actually spent in the SDPT3 optimization.
%                This is different from value in info.info because 
%                it includes all time SDPT3 takes to set up the 
%                optimization once its function is called.
%            See SDPT3 manual for more details. 
%
% See also: 
%
% Coding:   6/30/2001   Pete Seiler
%           7/13/2003   P. Seiler: Update for SDPT3 Ver3.0

function [copt,xopt,Zopt,info] = min_sdpt3(F0,Fi,blk,c,opt,x0,Z0)

% Is function called correctly?
if nargin <4 | nargout > 4
  error(['usage: [copt,xopt,Zopt,info] = ' ...
    'min_sdpt3(F0,Fi,blk,c,{opt},{x0},{Z0})']);
end
if nargin < 7
  Z0=[];
  if nargin <6
    x0=[];
    if nargin <5
      opt = [];
    end
  end
end

% Is SDPT3 installed and in Matlab's search path?
if exist('sqlp','file')~=2
  error( sprintf(['SDPT3 is not in Matlab''s search path. \n' ... 
  'To download SDPT3, go to: \n' ...
  '\t http://www.math.cmu.edu/~reha/sdpt3.html']));
end

% Convert from canonical form to SDPT3 form
blkin = blk;
blk = cell(1,2);
blk{1,1}='s'; blk{1,2}=blkin;

% Set SDPT3 options:  The setting of OPTIONS is stripped
% from parameters.m (an SDPT3 file).  parameters.m gets
% called from the SDPT3 startup.m file, but the following
% lines just ensure that these options are set.
OPTIONS.vers           = 1; 
OPTIONS.gam            = 0;  
OPTIONS.predcorr       = 1;
OPTIONS.expon          = [1 1]; 
OPTIONS.gaptol         = 1e-8;
OPTIONS.inftol         = 1e-8; 
OPTIONS.steptol        = 1e-6; 
OPTIONS.maxit          = 50; 
OPTIONS.printyes       = 1; 
OPTIONS.scale_data     = 0; 
OPTIONS.randnstate     = 0; 
OPTIONS.spdensity      = 0.5; 
OPTIONS.rmdepconstr    = 0; 
OPTIONS.cachesize      = 256; 
OPTIONS.linsys_options = 'raugmatsys'; 


fldnames = fieldnames(opt);
for i1 =1:length(fldnames)
  eval([fldnames{i1} '= opt.' fldnames{i1} ';']);
end
if ~isfield(opt,'printyes'); OPTIONS.printyes = opt.prt;  end
if ~isfield(opt,'gaptol');   OPTIONS.gaptol = opt.reltol; end
if ~isfield(opt,'maxit');    OPTIONS.maxit = opt.maxiter; end

X0=cell(1);X0{1}=eye(size(F0,1));
y0=zeros(length(c),1);
Z0=cell(1);Z0{1}=eye(size(F0,1));

% Solve with SDPT3
cputime0=cputime;
[obj,Zopt,xopt,slackvar,sdpt3info,gaphist,infeashist] = ...  
    sqlp(blk,Fi',F0,-c,OPTIONS,X0,y0,Z0);
info.solvertime = cputime-cputime0;

% Set some output variables
info.gaphist = gaphist;
info.infeashist = infeashist;
info.info = sdpt3info;

Zopt = full(Zopt{1});
copt = [c'*xopt -trace(F0*Zopt)];



